@@ -19,6 +19,8 @@
 #ifndef INDI_TELESCOPE_H
 #define INDI_TELESCOPE_H
 
+#include <string>
+
 #include <libnova.h>
 
 #include "defaultdevice.h"
@@ -33,12 +35,12 @@
    Implementing a basic telescope driver involves the child class performing the following steps:
    <ul>
    <li>The child class should define the telescope capabilities via the TelescopeCapability structure and sets in the default constructor.</li>
-   <li>If the telescope has additional properties, the child class should override initProperties and initilize the respective additional properties.</li>
+   <li>If the telescope has additional properties, the child class should override initProperties and initialize the respective additional properties.</li>
    <li>Once the parent class calls Connect(), the child class attempts to connect to the telescope and return either success of failure</li>
    <li>INDI::Telescope calls updateProperties() to enable the child class to define which properties to send to the client upon connection</li>
    <li>INDI::Telescope calls ReadScopeStatus() to check the link to the telescope and update its state and position. The child class should call newRaDec() whenever
    a new value is read from the telescope.</li>
-   <li>The child class should implmenet Goto() and Sync(), and Park()/UnPark() if applicable.</li>
+   <li>The child class should implement Goto() and Sync(), and Park()/UnPark() if applicable.</li>
    <li>INDI::Telescope calls disconnect() when the client request a disconnection. The child class should remove any additional properties it defined in updateProperties() if applicable</li>
    </ul>
 
@@ -141,6 +143,13 @@ class INDI::Telescope : public INDI::DefaultDevice
         */
         virtual bool Connect(const char *port, uint32_t baud);
 
+        /** \brief INDI::Telescope implementation of Connect() for TCP/IP connections.
+          \param hostname Host name or IP to connect to
+          \param port port
+          \return True if connection is successful, false otherwise
+          \warning Do not call this function directly, it is called by INDI::Telescope Connect() function.
+        */
+        virtual bool Connect(const char *hostname, const char *port);
 
         //Park
         /**
@@ -215,6 +224,12 @@ class INDI::Telescope : public INDI::DefaultDevice
          */
         void SetAxis2ParkDefault(double steps);
 
+        /**
+         * @brief isLocked is mount currently locked?
+         * @return true if lock status equals true and DomeClosedLockTP is Dome Locks or Dome Locks and Dome Parks (both).
+         */
+        bool isLocked();
+
         // Joystick helpers
         static void joystickHelper(const char * joystick_n, double mag, double angle, void *context);
         static void buttonHelper(const char * button_n, ISState state, void *context);
@@ -238,13 +253,13 @@ class INDI::Telescope : public INDI::DefaultDevice
         virtual bool ReadScopeStatus()=0;
 
         /** \brief Move the scope to the supplied RA and DEC coordinates
-            \return True if successful, false otherewise
+            \return True if successful, false otherwise
             \note This function is not implemented in INDI::Telescope, it must be implemented in the child class
         */
         virtual bool Goto(double ra,double dec)=0;
 
         /** \brief Set the telescope current RA and DEC coordinates to the supplied RA and DEC coordinates
-            \return True if successful, false otherewise
+            \return True if successful, false otherwise
             *\note This function implemented INDI::Telescope always returns false. Override the function to return true.
         */
         virtual bool Sync(double ra,double dec);
@@ -252,7 +267,7 @@ class INDI::Telescope : public INDI::DefaultDevice
         /** \brief Start or Stop the telescope motion in the direction dir.
          *  \param dir direction of motion
          *  \param command Start or Stop command
-            \return True if successful, false otherewise
+            \return True if successful, false otherwise
             \note This function is not implemented in INDI::Telescope, it must be implemented in the child class
         */
         virtual bool MoveNS(INDI_DIR_NS dir, TelescopeMotionCommand command);
@@ -260,25 +275,25 @@ class INDI::Telescope : public INDI::DefaultDevice
         /** \brief Move the telescope in the direction dir.
             \param dir direction of motion
             \param command Start or Stop command
-            \return True if successful, false otherewise
+            \return True if successful, false otherwise
             \note This function is not implemented in INDI::Telescope, it must be implemented in the child class
         */
         virtual bool MoveWE(INDI_DIR_WE dir, TelescopeMotionCommand command);
 
         /** \brief Park the telescope to its home position.
-            \return True if successful, false otherewise
+            \return True if successful, false otherwise
             *\note This function defaults to return false unless subclassed by the child class.
         */
         virtual bool Park();
 
         /** \brief Unpark the telescope if already parked.
-            \return True if successful, false otherewise
+            \return True if successful, false otherwise
             *\note This function defaults to return false unless subclassed by the child class.
         */
         virtual bool UnPark();
 
         /** \brief Abort telescope motion
-            \return True if successful, false otherewise
+            \return True if successful, false otherwise
             \note This function is not implemented in INDI::Telescope, it must be implemented in the child class
         */
         virtual bool Abort()=0;
@@ -286,7 +301,7 @@ class INDI::Telescope : public INDI::DefaultDevice
         /** \brief Update telescope time, date, and UTC offset.
          *  \param utc UTC time.
          *  \param utc_offset UTC offset in hours.
-            \return True if successful, false otherewise
+            \return True if successful, false otherwise
             \note This function performs no action unless subclassed by the child class if required.
         */
         virtual bool updateTime(ln_date *utc, double utc_offset);
@@ -295,7 +310,7 @@ class INDI::Telescope : public INDI::DefaultDevice
          *  \param latitude Site latitude in degrees.
          *  \param longitude Site latitude in degrees increasing eastward from Greenwich (0 to 360).
          *  \param elevation Site elevation in meters.
-            \return True if successful, false otherewise
+            \return True if successful, false otherwise
             \note This function performs no action unless subclassed by the child class if required.
         */
         virtual bool updateLocation(double latitude, double longitude, double elevation);        
@@ -332,6 +347,7 @@ class INDI::Telescope : public INDI::DefaultDevice
         //  We put the serial helper into the base telescope class
         //  One less piece to worry about in the hardware specific
         //  low level stuff
+        //  Mounts with ethernet can copy the socket fd into PortFD and override Connect and Disconnect.
         int PortFD;
 
         //  This is a variable filled in by the ReadStatus telescope
@@ -396,16 +412,31 @@ class INDI::Telescope : public INDI::DefaultDevice
         IText TimeT[2];
         ITextVectorProperty TimeTP;
 
-        // Active devices to snoop
+        // Active GPS/Dome device to snoop
         ITextVectorProperty ActiveDeviceTP;
-        IText ActiveDeviceT[1];
+        IText ActiveDeviceT[2];
+
+        // Switch to lock if dome is closed, and or force parking if dome parks
+        ISwitchVectorProperty DomeClosedLockTP;
+        ISwitch DomeClosedLockT[4];
+
+        // IP Address/Port
+        ITextVectorProperty AddressTP;
+        IText AddressT[2];
 
         ISwitch BaudRateS[6];
         ISwitchVectorProperty BaudRateSP;
 
+        // Lock Joystick Axis to one direciton only
+        ISwitch LockAxisS[2];
+        ISwitchVectorProperty LockAxisSP;
+
         uint32_t capability;
         int last_we_motion, last_ns_motion;
 
+        // Period in milliseconds to call ReadScopeStatus(). 1000 milliseconds by default
+        uint32_t updatePeriodMS = 1000;
+
         //Park
         char *LoadParkData();
         bool WriteParkData();        
@@ -415,7 +446,10 @@ private:
         bool processTimeInfo(const char *utc, const char *offset);
         bool processLocationInfo(double latitude, double longitude, double elevation);
 
+        void triggerSnoop(const char *driverName, const char *propertyName);
+
         TelescopeParkData parkDataType;
+        bool IsLocked;
         bool IsParked;        
         const char *ParkDeviceName;
         const char * Parkdatafile;
@@ -432,6 +466,9 @@ private:
 
         INDI::Controller *controller;
 
+        int sockfd = -1;
+        const uint8_t SOCKET_TIMEOUT = 5;
+
 };
 
 #endif // INDI::Telescope_H
